@@ -1,19 +1,17 @@
 /*!
   * Reqwest! A general purpose XHR connection manager
-  * (c) Dustin Diaz 2011
+  * (c) Dustin Diaz 2012
   * https://github.com/ded/reqwest
   * license MIT
   */
 !function (name, definition) {
   if (typeof module != 'undefined') module.exports = definition()
-  else if (typeof define == 'function' && define.amd) define(name, definition)
+  else if (typeof define == 'function' && define.amd) define(definition)
   else this[name] = definition()
 }('reqwest', function () {
 
-  var context = this
-    , win = window
+  var win = window
     , doc = document
-    , old = context.reqwest
     , twoHundo = /^20\d$/
     , byTag = 'getElementsByTagName'
     , readyState = 'readyState'
@@ -21,30 +19,32 @@
     , requestedWith = 'X-Requested-With'
     , head = doc[byTag]('head')[0]
     , uniqid = 0
+    , callbackPrefix = 'reqwest_' + (+new Date())
     , lastValue // data stored by the most recent JSONP callback
     , xmlHttpRequest = 'XMLHttpRequest'
-    , isArray = typeof Array.isArray == 'function' ? Array.isArray : function (a) {
-        return a instanceof Array
+
+  var isArray = typeof Array.isArray == 'function' ? Array.isArray : function (a) {
+    return a instanceof Array
+  }
+  var defaultHeaders = {
+      contentType: 'application/x-www-form-urlencoded'
+    , requestedWith: xmlHttpRequest
+    , accept: {
+        '*':  'text/javascript, text/html, application/xml, text/xml, */*'
+      , xml:  'application/xml, text/xml'
+      , html: 'text/html'
+      , text: 'text/plain'
+      , json: 'application/json, text/javascript'
+      , js:   'application/javascript, text/javascript'
       }
-    , defaultHeaders = {
-          contentType: 'application/x-www-form-urlencoded'
-        , accept: {
-              '*':  'text/javascript, text/html, application/xml, text/xml, */*'
-            , xml:  'application/xml, text/xml'
-            , html: 'text/html'
-            , text: 'text/plain'
-            , json: 'application/json, text/javascript'
-            , js:   'application/javascript, text/javascript'
-          }
-        , requestedWith: xmlHttpRequest
-      }
-    , xhr = win[xmlHttpRequest] ?
-        function () {
-          return new XMLHttpRequest()
-        } :
-        function () {
-          return new ActiveXObject('Microsoft.XMLHTTP')
-        }
+    }
+  var xhr = win[xmlHttpRequest] ?
+    function () {
+      return new XMLHttpRequest()
+    } :
+    function () {
+      return new ActiveXObject('Microsoft.XMLHTTP')
+    }
 
   function handleReadyState(o, success, error) {
     return function () {
@@ -69,6 +69,12 @@
     }
   }
 
+  function setCredentials(http, o) {
+    if (typeof o.withCredentials !== "undefined" && typeof http.withCredentials !== "undefined") {
+      http.withCredentials = !!o.withCredentials
+    }
+  }
+
   function generalCallback(data) {
     lastValue = data
   }
@@ -80,7 +86,8 @@
   function handleJsonp(o, fn, err, url) {
     var reqId = uniqid++
       , cbkey = o.jsonpCallback || 'callback' // the 'callback' key
-      , cbval = o.jsonpCallbackName || ('reqwest_' + reqId) // the 'callback' value
+      , cbval = o.jsonpCallbackName || reqwest.getcallbackPrefix(reqId)
+      // , cbval = o.jsonpCallbackName || ('reqwest_' + reqId) // the 'callback' value
       , cbreg = new RegExp('((^|\\?|&)' + cbkey + ')=([^&]+)')
       , match = url.match(cbreg)
       , script = doc.createElement('script')
@@ -102,11 +109,11 @@
     script.src = url
     script.async = true
     if (typeof script.onreadystatechange !== 'undefined') {
-        // need this for IE due to out-of-order onreadystatechange(), binding script
-        // execution to an event listener gives us control over when the script
-        // is executed. See http://jaubourg.net/2010/07/loading-script-as-onclick-handler-of.html
-        script.event = 'onclick'
-        script.htmlFor = script.id = '_reqwest_' + reqId
+      // need this for IE due to out-of-order onreadystatechange(), binding script
+      // execution to an event listener gives us control over when the script
+      // is executed. See http://jaubourg.net/2010/07/loading-script-as-onclick-handler-of.html
+      script.event = 'onclick'
+      script.htmlFor = script.id = '_reqwest_' + reqId
     }
 
     script.onload = script.onreadystatechange = function () {
@@ -147,6 +154,7 @@
     http = xhr()
     http.open(method, url, true)
     setHeaders(http, o)
+    setCredentials(http, o)
     http.onreadystatechange = handleReadyState(http, fn, err)
     o.before && o.before(http)
     http.send(data)
@@ -156,6 +164,7 @@
   function Reqwest(o, fn) {
     this.o = o
     this.fn = fn
+
     init.apply(this, arguments)
   }
 
@@ -165,10 +174,25 @@
   }
 
   function init(o, fn) {
+
     this.url = typeof o == 'string' ? o : o.url
     this.timeout = null
-    var type = o.type || setType(this.url)
-      , self = this
+
+    // whether request has been fulfilled for purpose
+    // of tracking the Promises
+    this._fulfilled = false
+    // success handlers
+    this._fulfillmentHandlers = []
+    // error handlers
+    this._errorHandlers = []
+    // complete (both success and fail) handlers
+    this._completeHandlers = []
+    this._erred = false
+    this._responseArgs = {}
+
+    var self = this
+      , type = o.type || setType(this.url)
+
     fn = fn || function () {}
 
     if (o.timeout) {
@@ -177,10 +201,30 @@
       }, o.timeout)
     }
 
+    if (o.success) {
+      this._fulfillmentHandlers.push(function () {
+        o.success.apply(o, arguments)
+      })
+    }
+
+    if (o.error) {
+      this._errorHandlers.push(function () {
+        o.error.apply(o, arguments)
+      })
+    }
+
+    if (o.complete) {
+      this._completeHandlers.push(function () {
+        o.complete.apply(o, arguments)
+      })
+    }
+
     function complete(resp) {
       o.timeout && clearTimeout(self.timeout)
       self.timeout = null
-      o.complete && o.complete(resp)
+      while (self._completeHandlers.length > 0) {
+        self._completeHandlers.shift()(resp)
+      }
     }
 
     function success(resp) {
@@ -200,17 +244,30 @@
         case 'html':
           resp = r
           break;
+        case 'xml':
+          resp = resp.responseXML;
+          break;
         }
       }
 
+      self._responseArgs.resp = resp
+      self._fulfilled = true
       fn(resp)
-      o.success && o.success(resp)
+      while (self._fulfillmentHandlers.length > 0) {
+        self._fulfillmentHandlers.shift()(resp)
+      }
 
       complete(resp)
     }
 
     function error(resp, msg, t) {
-      o.error && o.error(resp, msg, t)
+      self._responseArgs.resp = resp
+      self._responseArgs.msg = msg
+      self._responseArgs.t = t
+      self._erred = true
+      while (self._errorHandlers.length > 0) {
+        self._errorHandlers.shift()(resp, msg, t)
+      }
       complete(resp)
     }
 
@@ -225,6 +282,50 @@
   , retry: function () {
       init.call(this, this.o, this.fn)
     }
+
+    /**
+     * Small deviation from the Promises A CommonJs specification
+     * http://wiki.commonjs.org/wiki/Promises/A
+     */
+
+    /**
+     * `then` will execute upon successful requests
+     */
+  , then: function (success, fail) {
+      if (this._fulfilled) {
+        success(this._responseArgs.resp)
+      } else if (this._erred) {
+        fail(this._responseArgs.resp, this._responseArgs.msg, this._responseArgs.t)
+      } else {
+        this._fulfillmentHandlers.push(success)
+        this._errorHandlers.push(fail)
+      }
+      return this
+    }
+
+    /**
+     * `always` will execute whether the request succeeds or fails
+     */
+  , always: function (fn) {
+      if (this._fulfilled || this._erred) {
+        fn(this._responseArgs.resp)
+      } else {
+        this._completeHandlers.push(fn)
+      }
+      return this
+    }
+
+    /**
+     * `fail` will execute when the request fails
+     */
+  , fail: function (fn) {
+      if (this._erred) {
+        fn(this._responseArgs.resp, this._responseArgs.msg, this._responseArgs.t)
+      } else {
+        this._errorHandlers.push(fn)
+      }
+      return this
+    }
   }
 
   function reqwest(o, fn) {
@@ -239,7 +340,7 @@
   function serial(el, cb) {
     var n = el.name
       , t = el.tagName.toLowerCase()
-      , optCb = function(o) {
+      , optCb = function (o) {
           // IE gives value="" even where there is no value attribute
           // 'specified' ref: http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-862529273
           if (o && !o.disabled)
@@ -280,7 +381,7 @@
   function eachFormElement() {
     var cb = this
       , e, i, j
-      , serializeSubtags = function(e, tags) {
+      , serializeSubtags = function (e, tags) {
         for (var i = 0; i < tags.length; i++) {
           var fa = e[byTag](tags[i])
           for (j = 0; j < fa.length; j++) serial(fa[j], cb)
@@ -314,7 +415,7 @@
   // [ { name: 'name', value: 'value' }, ... ] style serialization
   reqwest.serializeArray = function () {
     var arr = []
-    eachFormElement.apply(function(name, value) {
+    eachFormElement.apply(function (name, value) {
       arr.push({name: name, value: value})
     }, arguments)
     return arr
@@ -356,7 +457,11 @@
     }
 
     // spaces should be + according to spec
-    return qs.replace(/&$/, '').replace(/%20/g,'+')
+    return qs.replace(/&$/, '').replace(/%20/g, '+')
+  }
+
+  reqwest.getcallbackPrefix = function (reqId) {
+    return callbackPrefix
   }
 
   // jQuery and Zepto compatibility, differences can be remapped here so you can call
@@ -372,4 +477,4 @@
   }
 
   return reqwest
-})
+});
