@@ -1,3 +1,4 @@
+/*global reqwest:true, sink:true, start:true, ender:true, v:true */
 !function (ajax) {
   var FakeXHR = function () {
     this.args = {}
@@ -43,10 +44,15 @@
     })
 
     test('JSONP', function (complete) {
+      // stub callback prefix
+      reqwest.getcallbackPrefix = function (id) {
+        return 'reqwest_' + id
+      }
       ajax({
         url: '/tests/fixtures/fixtures_jsonp.jsonp?callback=?',
         type: 'jsonp',
         success: function (resp) {
+          console.log('RESPONSE IS', resp)
           ok(resp, 'received response for unique generated callback')
           ok(resp && resp.boosh == "boosh", "correctly evaluated response for unique generated callback as JSONP")
           complete()
@@ -75,6 +81,31 @@
         }
       })
     })
+    test('XML', 4, function(){
+     ajax({
+       url:'/tests/fixtures/fixtures.xml',
+       type:'xml',
+       success:function(resp){
+         ok(resp &&resp instanceof Document, 'XML Response is a Document')
+         ok(resp && resp.documentElement && resp.documentElement.nodeName=='root', 'XML Response root is <root>')
+         ok(resp && resp.documentElement && resp.documentElement.hasChildNodes && resp.documentElement.firstChild.nodeName=='boosh' && resp.documentElement.firstChild.firstChild.nodeValue==='boosh', 'Correct XML response')
+       },
+       error:function(err){
+         ok(false, err.responseText)
+       }
+     })
+
+     ajax({
+      url:'/tests/fixtures/badfixtures.xml',
+      type:'xml',
+      success:function(resp){
+        ok(resp===null, 'No XML Response')
+      },
+      error:function(err){
+        ok(true, 'No XML Response')
+      }
+     })
+   })
   })
 
   sink('JSONP', function (test, ok) {
@@ -252,6 +283,65 @@
     })
   })
 
+  sink('Cross-origin Resource Sharing', function(test, ok) {
+
+    var supportsCors = window.XMLHttpRequest && ("withCredentials" in new window.XMLHttpRequest)
+
+    test('make request to another origin', 1, function() {
+      ajax({
+        url: 'http://localhost:5678/get-value',
+        type: 'text',
+        method: 'get',
+        crossOrigin: true,
+        complete: function (resp) {
+          if (supportsCors)
+            ok(resp.responseText === 'hello', 'request made successfully')
+          else
+            ok(true, 'browser does not support Cross-Origin Resource Sharing')
+        }
+      })
+    })
+
+    test('set cookie on other origin', 2, function() {
+      ajax({
+        url: 'http://localhost:5678/set-cookie',
+        type: 'text',
+        method: 'get',
+        crossOrigin: true,
+        withCredentials: true,
+        before: function (http) {
+          if (supportsCors)
+            ok(http.withCredentials === true, 'has set withCredentials on connection object')
+          else
+            ok(true, 'browser does not support Cross-Origin Resource Sharing')
+        },
+        complete: function (resp) {
+          if (supportsCors)
+            ok(resp.status === 200, 'cookie set successfully')
+          else
+            ok(true, 'browser does not support Cross-Origin Resource Sharing')
+        }
+      })
+    })
+
+    test('get cookie from other origin', 1, function() {
+      ajax({
+          url: 'http://localhost:5678/get-cookie-value',
+          type: 'text',
+          method: 'get',
+          crossOrigin: true,
+          withCredentials: true,
+          complete: function (resp) {
+            if (supportsCors)
+              ok(resp.responseText === 'hello', 'cookie value retrieved successfully')
+            else
+              ok(true, 'browser does not support Cross-Origin Resource Sharing')
+          }
+      })
+    })
+
+  })
+
   sink('Connection Object', function (test, ok) {
 
     test('setRequestHeaders', function (complete) {
@@ -534,8 +624,8 @@
         ok(sameValue(sh[name], value), 'serialize(' + str + ', {type:\'map\'})')
       } else {
         // the cases where an element shouldn't show up at all, checkbox not checked for example
-        ok(sa.length == 0, 'serialize(' + str + ', {type:\'array\'}) is []')
-        ok(v.keys(sh).length == 0, 'serialize(' + str + ', {type:\'map\'}) is {}')
+        ok(sa.length === 0, 'serialize(' + str + ', {type:\'array\'}) is []')
+        ok(v.keys(sh).length === 0, 'serialize(' + str + ', {type:\'map\'}) is {}')
       }
     }
 
@@ -547,7 +637,7 @@
 
     function executeMultiArgumentMethod(method, argType, options) {
       var els = [ foo, bar, choices ]
-        , ths = argType === BIND_ARGS ? els : null
+        , ths = argType === BIND_ARGS ? ender(els) : null
         , args = argType === PASS_ARGS ? els : []
 
       if (!!options) args.push(options)
@@ -662,7 +752,7 @@
     sHelper.reset()
 
     test('correctly serialize textarea', function (complete) {
-      textarea = sHelper.formElements(1, 'textarea', 0)
+      var textarea = sHelper.formElements(1, 'textarea', 0)
       // the texarea has 2 different newline styles, should come out as normalized CRLF as per forms spec
       ok('T3=%3F%0D%0AA+B%0D%0AZ' == ajax.serialize(textarea), 'serialize(textarea)')
       var sa = ajax.serialize(textarea, { type: 'array' })
@@ -921,17 +1011,17 @@
     })
 
     test('$(element, element, element...).serialize()', function (complete) {
-      sHelper.testMultiArgumentSerialize(ender._boosh.serialize, 'ender', BIND_ARGS)
+      sHelper.testMultiArgumentSerialize(ender.fn.serialize, 'ender', BIND_ARGS)
       complete()
     })
 
     test('$(element, element, element...).serialize({type:\'array\'})', function (complete) {
-      sHelper.testMultiArgumentSerializeArray(ender._boosh.serialize, 'ender', BIND_ARGS)
+      sHelper.testMultiArgumentSerializeArray(ender.fn.serialize, 'ender', BIND_ARGS)
       complete()
     })
 
     test('$(element, element, element...).serialize({type:\'map\'})', function (complete) {
-      sHelper.testMultiArgumentSerializeHash(ender._boosh.serialize, 'ender', BIND_ARGS)
+      sHelper.testMultiArgumentSerializeHash(ender.fn.serialize, 'ender', BIND_ARGS)
       complete()
     })
 
@@ -940,6 +1030,89 @@
     })
   })
 
+
+  /**
+   * Promise tests for `then` `fail` and `always`
+   */
+  sink('Promises', function (test, ok) {
+
+    test('always callback is called', function (complete) {
+      ajax({
+        url: '/tests/fixtures/fixtures.js'
+      })
+        .always(function () {
+          ok(true, 'called complete')
+          complete()
+        })
+    })
+
+    test('success and error handlers are called', 3, function () {
+      ajax({
+        url: '/tests/fixtures/invalidJSON.json',
+        type: 'json'
+      })
+        .then(function (resp) {
+          ok(false, 'success callback fired')
+        }, function (resp, msg) {
+          ok(msg == 'Could not parse JSON in response', 'error callback fired')
+        })
+
+      ajax({
+        url: '/tests/fixtures/invalidJSON.json',
+        type: 'json'
+      })
+        .fail(function (resp, msg) {
+          ok(msg == 'Could not parse JSON in response', 'fail callback fired')
+        })
+
+      ajax({
+        url: '/tests/fixtures/fixtures.json',
+        type: 'json'
+      })
+        .then(function (resp) {
+          ok(true, 'success callback fired')
+        }, function (resp) {
+          ok(false, 'error callback fired')
+        })
+    })
+
+    test('then and always handlers can be added after a response has been received', 2, function () {
+      var a = ajax({
+        url: '/tests/fixtures/fixtures.json',
+        type: 'json'
+      })
+        .always(function () {
+          setTimeout(function () {
+            a
+              .then(function () {
+                ok(true, 'success callback called')
+              }, function () {
+                ok(false, 'error callback called')
+              })
+              .always(function () {
+                ok(true, 'complete callback called')
+              })
+          }, 1)
+        })
+    })
+
+    test('failure handlers can be added after a response has been received', function (complete) {
+      var a = ajax({
+        url: '/tests/fixtures/invalidJSON.json',
+        type: 'json'
+      })
+        .always(function () {
+          setTimeout(function () {
+            a
+              .fail(function () {
+                ok(true, 'fail callback called')
+                complete()
+              })
+          }, 1)
+        })
+    })
+  })
+
   start()
 
 }(reqwest)
